import pymysql as mysql
import logging
import os

class MysqlHandler():

    def __init__(self, host, port, user, password):
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.checkDB(host=host, port=port, user=user, password=password)

    def checkDB(self, host, port, user, password):
        try:
            connection = mysql.connect(host=host, port=port, user=user, password=password, db='kubeai')
            self.checkTable(connection)
            connection.close()
        except Exception as e:
            if e.args[0] == 1049:
                connection = mysql.connect(host=host, port=port, user=user, password=password)
                create_db = "CREATE DATABASE kubeai"
                try:
                    with connection.cursor() as cursor:
                        cursor.execute(create_db)
                    connection.commit()
                    self.checkTable(connection)
                    connection.close()
                except Exception as e:
                    logging.error(str(e.args))
            else:
                logging.error(str(e.args))

    def checkTable(self, connection):
        check_table = "SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA='kubeai' AND TABLE_NAME='evaluate';"
        try:
            with connection.cursor() as cursor:
                cursor.execute(check_table)
                result = cursor.fetchall()
                if len(result) == 0:
                    create_table = '''
                    CREATE TABLE kubeai.evaluate(
                        id bigint(20) not null auto_increment primary key,
                        job_id varchar(50) not null,
                        name varchar(256),
                        namespace varchar(256),
                        uid varchar(256),
                        model_id varchar(256),
                        status varchar(32),
                        image varchar(256),
                        dataset_path varchar(256),
                        code varchar(256),
                        command varchar(256),
                        metrics text,
                        is_deleted tinyint(4),
                        report_path varchar(256),
                        gmt_created datetime,
                        gmt_modified datetime
                    );
                    '''
                    try:
                        with connection.cursor() as cursor:
                            cursor.execute(create_table)
                        connection.commit()
                    except Exception as e:
                        logging.error(str(e.args))
        except Exception as e:
            logging.error(str(e.args))

    def writeMetrics(self, metrics):
        connection = mysql.connect(host=self.host, port=self.port, user=self.user, password=self.password, db='kubeai')
        print(connection)
        self.job_id = os.getenv('JOB_ID')
        self.dataset_dir = os.getenv('DATASET_PATH')
        self.report_dir = os.getenv('METRICS_PATH')
        select_message = '''SELECT COUNT(*) as numbers FROM kubeai.evaluate WHERE job_id='%s';''' % str(self.job_id)
        try:
            with connection.cursor() as cursor:
                cursor.execute(select_message)
                result = cursor.fetchall()
                if result[0][0] == 0:
                    self.insertMetrics(connection, metrics)
                else:
                    self.updateMetrics(connection, metrics)
        except Exception as e:
            logging.error(str(e.args))

    def insertMetrics(self, connection, metrics):
        update_message = '''INSERT INTO kubeai.evaluate(
                                job_id,
                                dataset_path,
                                metrics,
                                report_path
                            ) VALUES (
                                "%s",
                                "%s",
                                "%s",
                                "%s"
                            );''' % (str(self.job_id) , self.dataset_dir, str(metrics), self.report_dir)
        try:
            with connection.cursor() as cursor:
                cursor.execute(update_message)
            connection.commit()
        except Exception as e:
            logging.error(str(e.args))

    def updateMetrics(self, connection, metrics):
        update_message = '''UPDATE kubeai.evaluate SET metrics = "%s" WHERE job_id="%s";''' % (str(metrics), str(self.job_id))
        try:
            with connection.cursor() as cursor:
                cursor.execute(update_message)
            connection.commit()
        except Exception as e:
            logging.error(str(e.args))


from abc import abstractmethod
import os
from kubeai.evaluate.mysql_handler import MysqlHandler

class Evaluator:

    def __init__(self):
        model_dir = os.getenv('MODEL_PATH')
        dataset_dir = os.getenv('DATASET_PATH')
        report_dir = os.getenv('METRICS_PATH')
        self.model_dir = model_dir
        self.dataset_dir = dataset_dir
        self.report_dir = report_dir

    def test_init(self, model_dir, dataset_dir, report_dir):
        self.model_dir = model_dir
        self.dataset_dir = dataset_dir
        self.report_dir = report_dir

    @abstractmethod
    def preprocess_dataset(self):
        pass

    @abstractmethod
    def load_model(self):
        pass

    @abstractmethod
    def evaluate_model(self, dataset):
        pass

    @abstractmethod
    def report_metrics(self, metrics):
        pass

    def report_process(self, metrics):
        self.metrics = metrics
        self.to_json_file()

        flag = os.getenv('ENABLE_MYSQL')
        if flag == "True":
            print("Writing mysql")
            host = os.getenv('MYSQL_HOST')
            port = int(os.getenv('MYSQL_PORT'))
            user = os.getenv('MYSQL_USERNAME')
            password = os.getenv('MYSQL_PASSWORD')

            handler = MysqlHandler(host=host, port=port, user=user, password=password)
            handler.writeMetrics(metrics=metrics)


    def to_json_file(self):
        import json
        with open(self.report_dir + '/metrics.json', "w") as f:
            f.write(json.dumps(self.metrics, ensure_ascii=False, indent=4, separators=(',', ':')))



from setuptools import setup, find_packages

setup(name='kubeai',
      version='0.1.0',
      description='Python SDK of Alibaba Cloud KubeAI',
      author='AlanFok',
      install_requires=["pymysql"],
      author_email='huozhixin.hzx@alibaba-inc.com',
      packages=find_packages()
)



# !/usr/bin/env python
# coding=utf-8

class KubeAI:

    @staticmethod
    def evaluate(evaluator):
        evaluator.load_model()
        dataset = evaluator.preprocess_dataset()
        metrics = evaluator.evaluate_model(dataset)
        evaluator.report_metrics(metrics)
        evaluator.report_process(metrics)

    @staticmethod
    def test(evaluator, model_dir, dataset_dir, report_dir):
        evaluator.test_init(model_dir=model_dir, dataset_dir=dataset_dir, report_dir=report_dir)
        evaluator.load_model()
        dataset = evaluator.preprocess_dataset()
        metrics = evaluator.evaluate_model(dataset)
        evaluator.report_metrics(metrics)
        evaluator.report_process(metrics)


import logging
import signal
from http.server import HTTPServer, SimpleHTTPRequestHandler
import threading

logging.basicConfig(format='[%(asctime)s] %(filename)s %(funcName)s():%(lineno)i [%(levelname)s] %(message)s',
                    level=logging.INFO)

g_alive = True


class KubeAIHandler(SimpleHTTPRequestHandler):

    def do_GET(self):
        paths = self.path
        logging.info(f"receive: {paths}")
        if paths == "/stop":
            global g_alive
            g_alive = False
        self.send_response(200, "ok")
        self.wfile.write(b"ok")


def HttpThread():
    logging.info("init http receiver")
    handler = KubeAIHandler
    handler.protocol_version = "HTTP/1.0"
    httpd = HTTPServer(('0.0.0.0', 9009), KubeAIHandler)
    httpd.serve_forever()


def init():
    logging.info("success init kubeai")
    try:
        t = threading.Thread(target=HttpThread)
        t.daemon = True
        t.start()
    except Exception as e:
        def handler_stop_signals(signum, frame):
            logging.error(f"recevie signal: {signum}")
            global g_alive
            g_alive = False

        logging.info("init signal receiver")
        signal.signal(signal.SIGTERM, handler_stop_signals)
    # try:
    #     import horovod.torch as hvd
    #     if hvd.local_rank() == 0:
    #         t = threading.Thread(target=HttpThread)
    #         t.daemon = True
    #         t.start()
    # except Exception as e:
    #     def handler_stop_signals(signum, frame):
    #         logging.error(f"recevie signal: {signum}")
    #         global g_alive
    #         g_alive = False
    #
    #     logging.info("init signal receiver")
    #     signal.signal(signal.SIGTERM, handler_stop_signals)


def check_alive():
    if not g_alive:
        logging.warning("kubeai is not alive")
    else:
        logging.debug("kubeai is alive")
    return g_alive
